/* Copyright 1990 by Carrick Sean Casey. */
/* For copying and distribution information, see the file COPYING. */

/* primitive to set fn internal variables */

#include "fn.h"
#include "externs.h"

/* struct for variable table */
typedef struct Variable {
	char *name;
	int type;
	char *address;
} VARIABLE;


/* variable types */
#define V_CHAR		1
#define V_BOOLEAN	2
#define V_NONNEG	3
#define V_INT		4
#define V_STRING	5


/* list of usable variables */
VARIABLE vars[] = {
	"beeps",	V_BOOLEAN,	(char *)&gv.beeps,
	"bufferlines",	V_NONNEG,	(char *)&gv.bufferlines,
	"commandchar",	V_CHAR,		(char *)&gv.cmdchar,
	"groupblanks",	V_BOOLEAN,	(char *)&gv.groupblanks,
	"history",	V_NONNEG,	(char *)&gv.phistory,
	"logfile",	V_STRING,	(char *)&gv.logfile,
	"pagesize",	V_INT,		(char *)&gv.pagesize,
	"pauseonshell",	V_BOOLEAN,	(char *)&gv.pauseonshell,
	"restricted",	V_BOOLEAN,	(char *)&gv.restricted,
	"timedisplay",	V_STRING,	(char *)&gv.timedisplay,
	"verifyquit",	V_BOOLEAN,	(char *)&gv.verifyquit,
	NULL, 0, NULL
};


fset(TARGLIST)
TARGDEFS
{
	static char *usage = "usage: fset var [value]";
	int x;

	if (argc == 1) {
		(void) listvars(NULL);
		return(TCL_OK);
	}

	else if (argc == 2) {
		if (listvars(argv[1]) < 0)
			TRETURNERR("fset: no such variable")
		else
			return(TCL_OK);
	}

	/* "restricted" is a special case */
	if (!strcmp(argv[1], "restricted")) {
		if (gv.restricted)
			TRETURNERR("fset: restricted mode can't be reset")
		else {
			restrict();
			return(TCL_OK);
		}
	}

	catargs(&argv[2]);

	for (x = 0; vars[x].name != NULL; x++) {
		if (strcmp(argv[1], vars[x].name) == 0) {
			if (setvar(interp, vars[x].type,
			vars[x].address, mbuf) < 0)
				return(TCL_ERROR);
			else
				break;
		}
	}

	if (vars[x].name == NULL) {
		sprintf(TRET, "fset: no such variable \"%s\"", argv[1]);
		return(TCL_ERROR);
	}

	return(TCL_OK);
}


setvar(interp, type, address, s)
Tcl_Interp *interp;
int type;
char *address;
char *s;
{
	int zero = 0, one = 1;
	int tmp;
	char *p, *malloc();

	switch(type) {
	case V_CHAR:
		*address = *s;
		break;
	case V_INT:
		tmp = atoi(s);
		bcopy(&tmp, address, sizeof(int));
		break;
	case V_NONNEG:
		tmp = atoi(s);
		if (tmp < 0) {
			strcpy(TRET, "fset: only non-negative values allowed");
			return(-1);
		}
		bcopy(&tmp, address, sizeof(int));
		break;
	case V_BOOLEAN:
		if (!strncmp(s, "on", 2) ||
		   !strncmp(s, "true", 4) ||
		   *s == 'y' || *s == '1')
			bcopy(&one, address, sizeof(int));
		else if (!strncmp(s, "off", 2) ||
			!strncmp(s, "false", 4) ||
			*s == 'n' || *s == '0')
				bcopy(&zero, address, sizeof(int));
		else {
			strcpy(TRET, "fset: only boolean values allowed");
			return(-1);
		}
		break;
	case V_STRING:
		tmp = strlen(s) + 1;
		if ((p = malloc(tmp)) == NULL) {
			strcpy(TRET, "fset: not enough memory to set string");
			return(-1);
		}
		bcopy(s, p, tmp);
		bcopy(&p, address, sizeof(p));
		break;
	}
	varsanity();
	return(0);
}

listvars(name)
char *name;
{
	int x;

	if (name == NULL) {
		putl("VARIABLE      TYPE  VALUE", PL_SCR);
		for (x = 0; vars[x].name != NULL; x++)
			printvar(vars[x].name, vars[x].type, vars[x].address);
		return(0);
	}

	else
		for (x = 0; vars[x].name != NULL; x++)
			if (strcmp(name, vars[x].name) == 0) {
				putl("VARIABLE      TYPE  VALUE", PL_SCR);
				printvar(vars[x].name,
				  vars[x].type, vars[x].address);
				break;
			}

	if (vars[x].name == NULL)
		return(-1);
	else
		return(0);
}


printvar(name, type, address)
char *name;
int type;
char *address;
{
	int ival;

	switch(type) {
	case V_CHAR:
		sprintf(mbuf, "%-12s  char  '%c'", name, *address);
		break;
	case V_INT:
		bcopy(address, &ival, sizeof(int));
		sprintf(mbuf, "%-12s  int   %d", name, ival);
		break;
	case V_NONNEG:
		bcopy(address, &ival, sizeof(int));
		sprintf(mbuf, "%-12s  nneg  %d", name, ival);
		break;
	case V_BOOLEAN:
		sprintf(mbuf, "%-12s  bool  %s", name,
		  (int)*address == 1 ? "true" : "false");
		break;
	case V_STRING:
		sprintf(mbuf, "%-12s  str   \"%s\"", name, *(char **)address);
		break;
	default:
		sprintf(mbuf, "printvar unknown type %d for \"%8s\"",type,name);
		break;
	}
	putl(mbuf, PL_SCR);
}

/* perform a sanity check on some variables */
varsanity()
{
	if (gv.pagesize < 0) {
		getwinsize();
		gv.pagesize = ttyinfo.rows - 1;
		if (gv.pagesize < 0)
			gv.pagesize = 0;
	}
	if (gv.pagesize > 500)
		gv.pagesize = 500;

	if (gv.bufferlines > 5000)
		gv.bufferlines = 5000;
}

restrict()
{
	gv.restricted = 1;
	tclrestrict();
}
